@@ -14,7 +14,8 @@ import typing
 from enum import Enum
 from functools import partial
 from ipaddress import IPv4Address, IPv4Interface, IPv4Network, IPv6Address, IPv6Interface, IPv6Network
-from typing import Any, Callable, Iterable, TypeVar
+from operator import attrgetter
+from typing import Any, Callable, Iterable, Literal, TypeVar
 
 import typing_extensions
 from pydantic_core import (
@@ -32,7 +33,7 @@ from pydantic.fields import FieldInfo
 from pydantic.types import Strict
 
 from ..config import ConfigDict
-from ..json_schema import JsonSchemaValue, update_json_schema
+from ..json_schema import JsonSchemaValue
 from . import _known_annotated_metadata, _typing_extra, _validators
 from ._core_utils import get_type_ref
 from ._internal_dataclass import slots_true
@@ -59,81 +60,72 @@ class SchemaTransformer:
 def get_enum_core_schema(enum_type: type[Enum], config: ConfigDict) -> CoreSchema:
     cases: list[Any] = list(enum_type.__members__.values())
 
-    if not cases:
-        # Use an isinstance check for enums with no cases.
-        # This won't work with serialization or JSON schema, but that's okay -- the most important
-        # use case for this is creating typevar bounds for generics that should be restricted to enums.
-        # This is more consistent than it might seem at first, since you can only subclass enum.Enum
-        # (or subclasses of enum.Enum) if all parent classes have no cases.
-        return core_schema.is_instance_schema(enum_type)
-
-    use_enum_values = config.get('use_enum_values', False)
-
-    if len(cases) == 1:
-        expected = repr(cases[0].value)
-    else:
-        expected = ', '.join([repr(case.value) for case in cases[:-1]]) + f' or {cases[-1].value!r}'
-
-    def to_enum(__input_value: Any) -> Enum:
-        try:
-            enum_field = enum_type(__input_value)
-            if use_enum_values:
-                return enum_field.value
-            return enum_field
-        except ValueError:
-            # The type: ignore on the next line is to ignore the requirement of LiteralString
-            raise PydanticCustomError('enum', f'Input should be {expected}', {'expected': expected})  # type: ignore
-
     enum_ref = get_type_ref(enum_type)
     description = None if not enum_type.__doc__ else inspect.cleandoc(enum_type.__doc__)
     if description == 'An enumeration.':  # This is the default value provided by enum.EnumMeta.__new__; don't use it
         description = None
-    updates = {'title': enum_type.__name__, 'description': description}
-    updates = {k: v for k, v in updates.items() if v is not None}
+    js_updates = {'title': enum_type.__name__, 'description': description}
+    js_updates = {k: v for k, v in js_updates.items() if v is not None}
 
-    def get_json_schema(_, handler: GetJsonSchemaHandler) -> JsonSchemaValue:
-        json_schema = handler(core_schema.literal_schema([x.value for x in cases], ref=enum_ref))
-        original_schema = handler.resolve_ref_schema(json_schema)
-        update_json_schema(original_schema, updates)
-        return json_schema
-
-    strict_python_schema = core_schema.is_instance_schema(enum_type)
-    if use_enum_values:
-        strict_python_schema = core_schema.chain_schema(
-            [strict_python_schema, core_schema.no_info_plain_validator_function(lambda x: x.value)]
-        )
-
-    to_enum_validator = core_schema.no_info_plain_validator_function(to_enum)
+    sub_type: Literal['str', 'int', 'float'] | None = None
     if issubclass(enum_type, int):
-        # this handles `IntEnum`, and also `Foobar(int, Enum)`
-        updates['type'] = 'integer'
-        lax = core_schema.chain_schema([core_schema.int_schema(), to_enum_validator])
-        # Disallow float from JSON due to strict mode
-        strict = core_schema.json_or_python_schema(
-            json_schema=core_schema.no_info_after_validator_function(to_enum, core_schema.int_schema()),
-            python_schema=strict_python_schema,
-        )
+        sub_type = 'int'
+        value_ser_type: core_schema.SerSchema = core_schema.simple_ser_schema('int')
     elif issubclass(enum_type, str):
         # this handles `StrEnum` (3.11 only), and also `Foobar(str, Enum)`
-        updates['type'] = 'string'
-        lax = core_schema.chain_schema([core_schema.str_schema(), to_enum_validator])
-        strict = core_schema.json_or_python_schema(
-            json_schema=core_schema.no_info_after_validator_function(to_enum, core_schema.str_schema()),
-            python_schema=strict_python_schema,
-        )
+        sub_type = 'str'
+        value_ser_type = core_schema.simple_ser_schema('str')
     elif issubclass(enum_type, float):
-        updates['type'] = 'numeric'
-        lax = core_schema.chain_schema([core_schema.float_schema(), to_enum_validator])
-        strict = core_schema.json_or_python_schema(
-            json_schema=core_schema.no_info_after_validator_function(to_enum, core_schema.float_schema()),
-            python_schema=strict_python_schema,
-        )
+        sub_type = 'float'
+        value_ser_type = core_schema.simple_ser_schema('float')
     else:
-        lax = to_enum_validator
-        strict = core_schema.json_or_python_schema(json_schema=to_enum_validator, python_schema=strict_python_schema)
-    return core_schema.lax_or_strict_schema(
-        lax_schema=lax, strict_schema=strict, ref=enum_ref, metadata={'pydantic_js_functions': [get_json_schema]}
-    )
+        # TODO this is an ugly hack, how do we trigger an Any schema for serialization?
+        value_ser_type = core_schema.plain_serializer_function_ser_schema(lambda x: x)
+
+    if cases:
+
+        def get_json_schema(schema: CoreSchema, handler: GetJsonSchemaHandler) -> JsonSchemaValue:
+            json_schema = handler(schema)
+            original_schema = handler.resolve_ref_schema(json_schema)
+            original_schema.update(js_updates)
+            return json_schema
+
+        # we don't want to add the missing to the schema if it's the default one
+        default_missing = getattr(enum_type._missing_, '__func__', None) == Enum._missing_.__func__  # type: ignore
+        enum_schema = core_schema.enum_schema(
+            enum_type,
+            cases,
+            sub_type=sub_type,
+            missing=None if default_missing else enum_type._missing_,
+            ref=enum_ref,
+            metadata={'pydantic_js_functions': [get_json_schema]},
+        )
+
+        if config.get('use_enum_values', False):
+            enum_schema = core_schema.no_info_after_validator_function(
+                attrgetter('value'), enum_schema, serialization=value_ser_type
+            )
+
+        return enum_schema
+
+    else:
+
+        def get_json_schema_no_cases(_, handler: GetJsonSchemaHandler) -> JsonSchemaValue:
+            json_schema = handler(core_schema.enum_schema(enum_type, cases, sub_type=sub_type, ref=enum_ref))
+            original_schema = handler.resolve_ref_schema(json_schema)
+            original_schema.update(js_updates)
+            return json_schema
+
+        # Use an isinstance check for enums with no cases.
+        # The most important use case for this is creating TypeVar bounds for generics that should
+        # be restricted to enums. This is more consistent than it might seem at first, since you can only
+        # subclass enum.Enum (or subclasses of enum.Enum) if all parent classes have no cases.
+        # We use the get_json_schema function when an Enum subclass has been declared with no cases
+        # so that we can still generate a valid json schema.
+        return core_schema.is_instance_schema(
+            enum_type,
+            metadata={'pydantic_js_functions': [get_json_schema_no_cases]},
+        )
 
 
 @dataclasses.dataclass(**slots_true)
@@ -283,30 +275,37 @@ def dequeue_validator(
         return collections.deque(handler(input_value), maxlen=maxlen)
 
 
+def serialize_sequence_via_list(
+    v: Any, handler: core_schema.SerializerFunctionWrapHandler, info: core_schema.SerializationInfo
+) -> Any:
+    items: list[Any] = []
+
+    mapped_origin = SEQUENCE_ORIGIN_MAP.get(type(v), None)
+    if mapped_origin is None:
+        # we shouldn't hit this branch, should probably add a serialization error or something
+        return v
+
+    for index, item in enumerate(v):
+        try:
+            v = handler(item, index)
+        except PydanticOmit:
+            pass
+        else:
+            items.append(v)
+
+    if info.mode_is_json():
+        return items
+    else:
+        return mapped_origin(items)
+
+
 @dataclasses.dataclass(**slots_true)
 class SequenceValidator:
     mapped_origin: type[Any]
     item_source_type: type[Any]
     min_length: int | None = None
     max_length: int | None = None
-    strict: bool = False
-
-    def serialize_sequence_via_list(
-        self, v: Any, handler: core_schema.SerializerFunctionWrapHandler, info: core_schema.SerializationInfo
-    ) -> Any:
-        items: list[Any] = []
-        for index, item in enumerate(v):
-            try:
-                v = handler(item, index)
-            except PydanticOmit:
-                pass
-            else:
-                items.append(v)
-
-        if info.mode_is_json():
-            return items
-        else:
-            return self.mapped_origin(items)
+    strict: bool | None = None
 
     def __get_pydantic_core_schema__(self, source_type: Any, handler: GetCoreSchemaHandler) -> CoreSchema:
         if self.item_source_type is Any:
@@ -341,7 +340,10 @@ class SequenceValidator:
             else:
                 coerce_instance_wrap = partial(core_schema.no_info_after_validator_function, self.mapped_origin)
 
-            constrained_schema = core_schema.list_schema(items_schema, **metadata)
+            # we have to use a lax list schema here, because we need to validate the deque's
+            # items via a list schema, but it's ok if the deque itself is not a list (same for Counter)
+            metadata_with_strict_override = {**metadata, 'strict': False}
+            constrained_schema = core_schema.list_schema(items_schema, **metadata_with_strict_override)
 
             check_instance = core_schema.json_or_python_schema(
                 json_schema=core_schema.list_schema(),
@@ -349,7 +351,7 @@ class SequenceValidator:
             )
 
             serialization = core_schema.wrap_serializer_function_ser_schema(
-                self.serialize_sequence_via_list, schema=items_schema or core_schema.any_schema(), info_arg=True
+                serialize_sequence_via_list, schema=items_schema or core_schema.any_schema(), info_arg=True
             )
 
             strict = core_schema.chain_schema([check_instance, coerce_instance_wrap(constrained_schema)])
